@@ -26,6 +26,11 @@ JZ 20210321 cleanup t_comport struct
 JZ 20210321 use (int) instead of t_float for members where applicable
 JZ 20210321 allow the user to turn ON input processing (again)
 JZ 20210321 allow the user to specify a device pattern as creation argument
+
+JZ 20240926 extend valid baudrates
+JZ 20240926 negative port-numbers at creation time indicate not-to-open
+AR 20241008 allow to query only USB devices or ports
+AR 20241008 allow to lock the opened device (Linux / MacOS)
 */
 
 #include "m_pd.h"
@@ -43,6 +48,7 @@ JZ 20210321 allow the user to specify a device pattern as creation argument
 #include <fcntl.h>
 #include <sys/ioctl.h> /* for ioctl DTR */
 #include <termios.h> /* for TERMIO ioctl calls */
+#include <sys/file.h> /* for flock() */
 #include <unistd.h>
 #include <glob.h>
 #define HANDLE int
@@ -75,10 +81,13 @@ typedef struct comport
     struct termios  oldcom_termio; /* save the old com config */
     struct termios  com_termio; /* for the new com config */
 #endif
+    t_bool          x_lock; /* the file descriptor has to be locked when opened */
+    t_bool          x_locked; /* the file descriptor has been locked */
 
   /* device specifications */
     t_symbol        *serial_device;
-    char            serial_device_prefix[MAXPDSTRING];/* the device name without the number */
+    const char*     pretty_name;
+    const char      *serial_device_prefix;/* the device name without the number */
     short           comport; /* holds the comport # */
 
   /* device configuration */
@@ -158,70 +167,89 @@ static long baudspeedbittable[] =
 #ifdef  IRIX
 #define OPENPARAMS (O_RDWR|O_NDELAY|O_NOCTTY)
 #define TIONREAD FIONREAD         /* re map the IOCTL function */
-#define BAUDRATE_230400 -1
-#define BAUDRATE_115200 -1
-#define BAUDRATE_57600  -1
-#define BAUDRATE_38400  B38400
 #else /* IRIX */
 #define OPENPARAMS (O_RDWR|O_NDELAY|O_NOCTTY)
-#define BAUDRATE_230400 B230400
-#define BAUDRATE_115200 B115200
-#define BAUDRATE_57600  B57600
-#define BAUDRATE_38400  B38400
 #endif /* else IRIX */
 
+typedef struct baudbits_ {
+  long rate;
+  long speedbits;
+} baudbits_t;
+
 static
-long baudspeedbittable[] =
-{
-    BAUDRATE_230400,
-    BAUDRATE_115200,  /* CPU SPECIFIC */
-    BAUDRATE_57600,   /* CPU SPECIFIC */
-    BAUDRATE_38400,   /* CPU SPECIFIC */
-    B19200,
-    B9600,
-    B4800,
-    B2400,
-    B1800,
-    B1200,
-    B600,
-    B300,
-    B200,
-    B150,
-    B134,
-    B110,
-    B75,
-    B50,
-    B0
+baudbits_t baudbitstable[] = {
+#ifdef B4000000
+  {4000000, B4000000},
+#endif
+#ifdef B3500000
+  {3500000, B3500000},
+#endif
+#ifdef B3000000
+  {3000000, B3000000},
+#endif
+#ifdef B2500000
+  {2500000, B2500000},
+#endif
+#ifdef B2000000
+  {2000000, B2000000},
+#endif
+#ifdef B1500000
+  {1500000, B1500000},
+#endif
+#ifdef B1152000
+  {1152000, B1152000},
+#endif
+#ifdef B1000000
+  {1000000, B1000000},
+#endif
+#ifdef B921600
+  {921600, B921600},
+#endif
+#ifdef B576000
+  {576000, B576000},
+#endif
+#ifdef B500000
+  {500000, B500000},
+#endif
+#ifdef B460800
+  {460800, B460800},
+#endif
+#ifdef B230400
+  {230400, B230400},
+#else
+  /* previously, this was supported without an #ifdef */
+# warning baudrate 230400 not supported (anymore)?
+#endif
+#ifdef B115200
+  {115200, B115200},
+#endif
+#ifdef B57600
+  {57600, B57600},
+#endif
+#ifdef B38400
+  {38400, B38400},
+#endif
+  {19200, B19200},
+  {9600, B9600},
+  {4800, B4800},
+  {2400, B2400},
+  {1800, B1800},
+  {1200, B1200},
+  {600, B600},
+  {300, B300},
+  {200, B200},
+  {150, B150},
+  {134, B134},
+  {110, B110},
+  {75, B75},
+  {50, B50},
+  {0, B0}
 };
 
+
+
 struct timeval null_tv;
 
-#define BAUDRATETABLE_LEN 19
-
-static long baudratetable[] =
-{
-    230400L,
-    115200L,
-    57600L,
-    38400L,
-    19200L,
-    9600L,
-    4800L,
-    2400L,
-    1800L,
-    1200L,
-    600L,
-    300L,
-    200L,
-    150L,
-    134L,
-    110L,
-    75L,
-    50L,
-    0L
-
-}; /* holds the baud rate selections */
-
 #endif /* else _WIN32 */
 
 /* From man cfsetospeed:
@@ -312,10 +340,10 @@ static void comport_output_rtscts(t_comport *x);
 static void comport_output_xonxoff(t_comport *x);
 static void comport_output_hupcl(t_comport *x);
 static void comport_output_rxerrors(t_comport *x);
-static void comport_enum(t_comport *x);
+static void comport_enum(t_comport *x, t_bool to_console, int argc, t_atom *argv);
 static void comport_info(t_comport *x);
-static void comport_devices(t_comport *x);
-static void comport_ports(t_comport *x);
+static void comport_devices(t_comport *x, t_symbol *s, int argc, t_atom *argv);
+static void comport_ports(t_comport *x, t_symbol *s, int argc, t_atom *argv);
 static void comport_set_verbose(t_comport *x, t_floatarg f);
 static void comport_help(t_comport *x);
 void comport_setup(void);
@@ -326,6 +354,22 @@ void comport_setup(void);
 
 /* ------------ sys dependent serial setup helpers ---------------- */
 
+#ifdef _WIN32
+/* According to http://msdn2.microsoft.com/en-us/library/aa363858.aspx To
+specify a COM port number greater than 9, use the following syntax:
+"\\\\.\\COM10".  This syntax works for all port numbers and hardware that
+allows COM port numbers to be specified. */
+    static const char SERIAL_DEVICE_PREFIX[] = "\\\\.\\COM";
+/* for UNIX, this is a glob pattern for matching devices  */
+#elif defined __APPLE__
+    static const char SERIAL_DEVICE_PREFIX[] = "/dev/tty.*";
+#elif defined __linux__
+    /* serial: ttyS?    USB-serial: ttyUSB?   USB-CDC: ttyACM? */
+    static const char SERIAL_DEVICE_PREFIX[] = "/dev/tty[ASU]*";
+#elif defined IRIX
+    static const char SERIAL_DEVICE_PREFIX[] = "/dev/ttyd*";
+#endif
+
 
 /* --------------------- NT ------------------------------------ */
 
@@ -486,6 +530,7 @@ static int set_hupcl(t_comport *x, int nr)
 
 static HANDLE open_serial(unsigned int com_num, t_comport *x)
 {
+    const char* pretty_name = 0;
     HANDLE          fd;
     COMMTIMEOUTS    timeouts;
     char            buffer[MAX_PATH];
@@ -517,7 +562,10 @@ static HANDLE open_serial(unsigned int com_num, t_comport *x)
 #endif
         x->serial_device = gensym(buffer);
     }
-    comport_verbose("[comport] Opening %s", &x->serial_device->s_name[4]);/* skip slashes and dot */
+    if(x->serial_device && x->serial_device->s_name)
+        pretty_name = &x->serial_device->s_name[4];/* skip slashes and dot */
+
+    comport_verbose("[comport] Opening %s", pretty_name);
     fd = CreateFileA( x->serial_device->s_name,
         GENERIC_READ | GENERIC_WRITE,
         0,
@@ -551,7 +599,7 @@ static HANDLE open_serial(unsigned int com_num, t_comport *x)
                 break;
         }
         pd_error(x, "[comport]: could not open device %s:\n failure(%ld) %s\n",
-        &x->serial_device->s_name[4], dw, errStr);
+            pretty_name, dw, errStr);
         return INVALID_HANDLE_VALUE;
     }
 
@@ -559,8 +607,7 @@ static HANDLE open_serial(unsigned int com_num, t_comport *x)
 
     if (!GetCommState(fd, &(x->dcb_old)))
     {
-        pd_error(x, "[comport]: could not get old dcb of device %s\n",
-            &x->serial_device->s_name[4]);
+        pd_error(x, "[comport]: could not get old dcb of device %s\n", pretty_name);
         CloseHandle(fd);
         return INVALID_HANDLE_VALUE;
     }
@@ -569,8 +616,7 @@ static HANDLE open_serial(unsigned int com_num, t_comport *x)
 
     if (!GetCommState(fd, &(x->dcb)))
     {
-        pd_error(x, "[comport]: could not get new dcb of device %s\n",
-            &x->serial_device->s_name[4]);
+        pd_error(x, "[comport]: could not get new dcb of device %s\n", pretty_name);
 
         CloseHandle(fd);
         return INVALID_HANDLE_VALUE;
@@ -611,12 +657,11 @@ static HANDLE open_serial(unsigned int com_num, t_comport *x)
     if(set_serial(x))
     {
         comport_verbose("[comport] opened serial line device %d (%s)\n",
-            com_num, &x->serial_device->s_name[4]);
+            com_num, pretty_name);
     }
     else
     {
-        pd_error(x, "[comport] could not set params to control dcb of device %s\n",
-            &x->serial_device->s_name[4]);
+        pd_error(x, "[comport] could not set params to control dcb of device %s\n", pretty_name);
         CloseHandle(fd);
         return INVALID_HANDLE_VALUE;
     }
@@ -643,6 +688,7 @@ static HANDLE open_serial(unsigned int com_num, t_comport *x)
         pd_error(x, "[comport] Couldn't do SetupComm (%ld)", GetLastError());
     }
     x->comport = com_num;/* output on next tick */
+    x->pretty_name = pretty_name;
     return fd;
 }
 
@@ -653,14 +699,14 @@ static HANDLE close_serial(t_comport *x)
         if (!SetCommState(x->comhandle, &(x->dcb_old)))
         {
             pd_error(x, "[comport] ** ERROR ** couldn't reset params to DCB of device %s\n",
-            &x->serial_device->s_name[4]);
+                x->pretty_name);
         }
         if (!SetCommTimeouts(x->comhandle, &(x->old_timeouts)))
         {
             pd_error(x, "[comport] Couldn't reset old_timeouts for serial device");
         }
         CloseHandle(x->comhandle);
-        comport_verbose("[comport] closed %s", &x->serial_device->s_name[4]);
+        comport_verbose("[comport] closed %s", x->pretty_name);
     }
     return INVALID_HANDLE_VALUE;
 }
@@ -705,27 +751,34 @@ int comport_get_cts(t_comport *x)
     return cts_state;
 }
 
+static void check_lock(t_comport *x)
+{
+	 /* on windows, this is a no-op */
+	(void)x;
+}
 #else /* NT */
 /* ----------------- POSIX - UNIX ------------------------------ */
 
 static long get_baud_ratebits(t_comport *x, long *baud)
 {
-    int i = 0;
+    unsigned int i = 0;
+    const unsigned int tablelen = sizeof(baudbitstable) / sizeof(*baudbitstable);
 
-    while(i < BAUDRATETABLE_LEN && baudratetable[i] > *baud) i++;
+    while(i < tablelen && baudbitstable[i].rate > *baud) i++;
 
-    if(baudratetable[i] != *baud)
-        pd_error(x, "[comport]: %ld not valid, using closest value: %ld", *baud, baudratetable[i]);
+    if(baudbitstable[i].rate != *baud)
+        pd_error(x, "[comport]: %ld not valid, using closest value: %ld", *baud, baudbitstable[i].rate);
 
     /* nearest Baudrate finding */
-    if(i==BAUDRATETABLE_LEN ||  baudspeedbittable[i] < 0)
+    if(i==tablelen ||  baudbitstable[i].rate < 0)
     {
         pd_error(x, "*Warning* The baud rate %ld is not supported or out of range, using 9600\n",*baud);
-        i = 8;
+	*baud = 9600;
+	return B9600;
     }
-    *baud =  baudratetable[i];
 
-    return baudspeedbittable[i];
+    *baud =  baudbitstable[i].rate;
+    return baudbitstable[i].speedbits;
 }
 
 static int set_baudrate(t_comport *x, int ibaud)
@@ -734,7 +787,7 @@ static int set_baudrate(t_comport *x, int ibaud)
     long            baud = ibaud;
     long        baudbits = get_baud_ratebits(x, &baud);
 
-    comport_verbose("[comport] set_baudrate: Setting baud rate to %g with baudbits 0x%X", baud, baudbits);
+    comport_verbose("[comport] set_baudrate: Setting baud rate to %ld with baudbits 0x%X", baud, baudbits);
     if( cfsetispeed(tio, baudbits) != 0 )
         pd_error(x, "[comport]: ERROR failed to set bitrate: %ld", baudbits);
     if( cfsetospeed(tio, baudbits) != 0 )
@@ -893,6 +946,30 @@ static int set_break(t_comport *x, int on)
     return ((status < 0)? status: (on != 0));
 }
 
+static t_bool lock_fd(t_comport *x, int fd, int lock)
+{
+    int operation = (lock ? LOCK_EX : LOCK_UN) | LOCK_NB;
+    int ret;
+
+    if((lock == x->x_locked) || (fd == INVALID_HANDLE_VALUE)) return 1;
+    ret = flock(fd, operation);
+    if(ret == 0) {
+        x->x_locked = lock;
+    }
+    return (ret == 0);
+}
+
+static void check_lock(t_comport *x)
+{
+    if(x->x_lock) {
+        if(!lock_fd(x, x->comhandle, 1)) {
+            comport_verbose("[comport] file descriptor of %s is already locked, closing!\n",
+                x->serial_device->s_name);
+            comport_close(x);
+        }
+    }
+}
+
 static int open_serial(unsigned int com_num, t_comport *x)
 {
     int             fd;
@@ -949,6 +1026,8 @@ static int open_serial(unsigned int com_num, t_comport *x)
     }
     globfree( &(glob_buffer) );
 
+    //check_file_availability(x->serial_device->s_name);
+
     if((fd = open(x->serial_device->s_name, OPENPARAMS)) == INVALID_HANDLE_VALUE)
     {
         pd_error(x, "[comport] ** ERROR ** could not open device %s:\n failure(%d): %s\n",
@@ -1021,6 +1100,8 @@ static int open_serial(unsigned int com_num, t_comport *x)
         return INVALID_HANDLE_VALUE;
     }
     x->comport = com_num; /* output at next comport_tick */
+    x->pretty_name = x->serial_device->s_name;
+
     return fd;
 }
 
@@ -1031,6 +1112,7 @@ static int close_serial(t_comport *x)
 
     if(fd != INVALID_HANDLE_VALUE)
     {
+        lock_fd(x, fd, 0);
         tcsetattr(fd, TCSANOW, tios);
         close(fd);
         comport_verbose("[comport] closed port %i (%s)", x->comport, x->serial_device->s_name);
@@ -1330,32 +1412,15 @@ static void *comport_new(t_symbol *s, int argc, t_atom *argv)
     t_comport *x;
     HANDLE    fd;
     const char *serial_device_prefix;
-    t_float com_num = 0;
+    int com_num = 0;
     int ibaud = 9600;
     (void)s; /* squelch unused-parameter warning */
 
-#ifdef _WIN32
-/* According to http://msdn2.microsoft.com/en-us/library/aa363858.aspx To
-specify a COM port number greater than 9, use the following syntax:
-"\\\\.\\COM10".  This syntax works for all port numbers and hardware that
-allows COM port numbers to be specified. */
-    serial_device_prefix = "\\\\.\\COM";
-#endif /* _WIN32 */
-/* for UNIX, this is a glob pattern for matching devices  */
-#ifdef __APPLE__
-    serial_device_prefix = "/dev/tty.*";
-#endif /* __APPLE__ */
-#ifdef IRIX
-    serial_device_prefix = "/dev/ttyd*";
-#endif /* IRIX */
-#ifdef __linux__
-    /* serial: ttyS?    USB-serial: ttyUSB?   USB-CDC: ttyACM? */
-    serial_device_prefix = "/dev/tty[ASU]*";
-#endif /* __linux__ */
+    serial_device_prefix = SERIAL_DEVICE_PREFIX;
 
     if(argc > 0) {
       if (argv->a_type == A_FLOAT)
-        com_num = atom_getfloatarg(0, argc, argv);
+        com_num = (int)atom_getfloatarg(0, argc, argv);
       else
         serial_device_prefix = atom_getsymbol(argv)->s_name;
     }
@@ -1364,11 +1429,8 @@ allows COM port numbers to be specified. */
 
 /*	 Open the Comport for RD and WR and get a handle */
 /* this line should use a real serial device */
-#ifdef _WIN32
-    strncpy_s(test.serial_device_prefix, strlen(serial_device_prefix) + 1, serial_device_prefix, strlen(serial_device_prefix) + 1);
-#else
-    strncpy(test.serial_device_prefix, serial_device_prefix, MAXPDSTRING);
-#endif
+
+    test.serial_device_prefix = serial_device_prefix;
     test.baud = ibaud;
     test.data_bits = 8; /* default 8 data bits */
     test.parity_bit = 0;/* default no parity bit */
@@ -1380,17 +1442,21 @@ allows COM port numbers to be specified. */
     test.ctsrts = 0; /* default no hardware handshaking */
     test.xonxoff = 0; /* default no software handshaking */
     test.hupcl = 1; /* default hangup on close */
-    fd = open_serial((unsigned int)com_num, &test);
+    test.x_verbose = 0; /* disable verbose for the 'test' comport */
+
+    /* don't try to open negative devices */
+    if(com_num < 0) {
+      fd = INVALID_HANDLE_VALUE;
+    } else {
+      fd = open_serial((unsigned int)com_num, &test);
+    }
 
     /* Now  nothing really bad could happen so we create the class */
     x = (t_comport *)pd_new(comport_class);
 
     x->comport = test.comport;/* com_num */
-#ifdef _WIN32
-    strncpy_s(x->serial_device_prefix, strlen(serial_device_prefix) + 1, serial_device_prefix, strlen(serial_device_prefix) + 1);
-#else
-    strncpy(x->serial_device_prefix, serial_device_prefix, MAXPDSTRING);
-#endif
+    x->serial_device_prefix = serial_device_prefix;
+
     x->serial_device = test.serial_device; /* we need this so 'help' doesn't crash */
 
     x->baud = test.baud;
@@ -1402,9 +1468,9 @@ allows COM port numbers to be specified. */
     x->hupcl = test.hupcl;
     x->comhandle = fd; /* holds the comport handle */
 
-    if(fd == INVALID_HANDLE_VALUE )
+    if(fd == INVALID_HANDLE_VALUE && com_num>=0)
     {
-        pd_error(x, "[comport] opening serial port %g failed!", com_num);
+        pd_error(x, "[comport] opening serial port %d failed!", com_num);
     }
     else
     {
@@ -1454,6 +1520,7 @@ allows COM port numbers to be specified. */
 
     x->x_verbose = 0;
     x->x_inprocess = 0;
+    x->x_lock = x->x_locked = 0;
 
     return x;
 }
@@ -1485,19 +1552,9 @@ static void comport_baud(t_comport *x,t_floatarg f)
 
     if(set_serial(x) == 0)
     {
-        pd_error(x,"[comport] ** ERROR ** could not set baudrate of device %s\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4]);
-#else
-            x->serial_device->s_name);
-#endif
+        pd_error(x,"[comport] ** ERROR ** could not set baudrate of device %s\n", x->pretty_name);
     }
-    else comport_verbose("[comport] set baudrate of %s to %d\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4], x->baud);
-#else
-            x->serial_device->s_name, x->baud);
-#endif
+    else comport_verbose("[comport] set baudrate of %s to %d\n", x->pretty_name, x->baud);
 }
 
 static void comport_bits(t_comport *x,t_floatarg fbits)
@@ -1508,20 +1565,10 @@ static void comport_bits(t_comport *x,t_floatarg fbits)
 
     if(set_serial(x) == 0)
     {
-        pd_error(x,"[comport] ** ERROR ** could not set bits of device %s\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4]);
-#else
-            x->serial_device->s_name);
-#endif
+        pd_error(x,"[comport] ** ERROR ** could not set bits of device %s\n", x->pretty_name);
         return;
     }
-    else comport_verbose("[comport] set bits of %s to %d\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4], bits);
-#else
-            x->serial_device->s_name, bits);
-#endif
+    else comport_verbose("[comport] set bits of %s to %d\n", x->pretty_name, bits);
     x->data_bits = bits;
 }
 
@@ -1534,48 +1581,34 @@ static void comport_parity(t_comport *x,t_floatarg fparity)
 
     if(set_serial(x) == 0)
     {
-        pd_error(x,"[comport] ** ERROR ** could not set extra paritybit of device %s\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4]);
-#else
-            x->serial_device->s_name);
-#endif
+        pd_error(x,"[comport] ** ERROR ** could not set extra paritybit of device %s\n", x->pretty_name);
         return;
     }
-    else comport_verbose("[comport] set extra paritybit of %s to %d\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4], parity);
-#else
-            x->serial_device->s_name, parity);
-#endif
+    else comport_verbose("[comport] set extra paritybit of %s to %d\n", x->pretty_name, parity);
     x->parity_bit = parity;
 }
 
 static void comport_stopbit(t_comport *x, t_floatarg fstop)
 {
     t_float stop = set_stopflag(x, fstop);
+    const char*stopname =
+#ifdef _WIN32
+      "stopbits"
+#else
+      "extra stopbit"
+#endif
+      ;
+
 
     if(x->comhandle == INVALID_HANDLE_VALUE)return;
 
     if(set_serial(x) == 0)
     {
-#ifdef _WIN32
-        pd_error(x,"[comport] ** ERROR ** could not set stopbits of device %s to %g\n",
-            &x->serial_device->s_name[4], stop);
-#else
-        pd_error(x,"[comport] ** ERROR ** could not set extra stopbit of device %s\n",
-            x->serial_device->s_name);
-#endif
+        pd_error(x,"[comport] ** ERROR ** could not set %s of device %s\n",
+            stopname, x->pretty_name);
         return;
     }
-    else comport_verbose
-#ifdef _WIN32
-        ("[comport] set stopbits of %s to %g\n",
-            &x->serial_device->s_name[4], stop);
-#else
-        ("[comport] set extra stopbit of %s to %g\n",
-            x->serial_device->s_name, stop);
-#endif
+    else comport_verbose("[comport] set %s of %s to %g\n", stopname, x->pretty_name, stop);
     x->stop_bits = stop;
 }
 
@@ -1587,20 +1620,10 @@ static void comport_rtscts(t_comport *x,t_floatarg f)
 
     if(set_serial(x) == 0)
     {
-        pd_error(x,"[comport] ** ERROR ** could not set rts_cts of device %s\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4]);
-#else
-            x->serial_device->s_name);
-#endif
+        pd_error(x,"[comport] ** ERROR ** could not set rts_cts of device %s\n", x->pretty_name);
         return;
     }
-    else comport_verbose("[comport] set rts-cts of %s to %g\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4], f);
-#else
-            x->serial_device->s_name, f);
-#endif
+    else comport_verbose("[comport] set rts-cts of %s to %g\n", x->pretty_name, f);
     x->ctsrts = f;
 }
 
@@ -1612,19 +1635,9 @@ static void comport_dtr(t_comport *x,t_floatarg f)
 
     if(f < 0)
     {
-        pd_error(x,"[comport] ** ERROR ** could not set dtr of device %s\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4]);
-#else
-            x->serial_device->s_name);
-#endif
+        pd_error(x,"[comport] ** ERROR ** could not set dtr of device %s\n", x->pretty_name);
     }
-    else comport_verbose("[comport] set dtr of %s to %g\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4], f);
-#else
-            x->serial_device->s_name, f);
-#endif
+    else comport_verbose("[comport] set dtr of %s to %g\n", x->pretty_name, f);
 }
 
 static void comport_rts(t_comport *x,t_floatarg f)
@@ -1635,19 +1648,9 @@ static void comport_rts(t_comport *x,t_floatarg f)
 
     if(f < 0)
     {
-        pd_error(x,"[comport] ** ERROR ** could not set rts of device %s\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4]);
-#else
-            x->serial_device->s_name);
-#endif
+        pd_error(x,"[comport] ** ERROR ** could not set rts of device %s\n", x->pretty_name);
     }
-    else comport_verbose("[comport] set rts of %s to %g\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4], f);
-#else
-            x->serial_device->s_name, f);
-#endif
+    else comport_verbose("[comport] set rts of %s to %g\n", x->pretty_name, f);
 }
 
 static void comport_break(t_comport *x,t_floatarg f)
@@ -1658,19 +1661,9 @@ static void comport_break(t_comport *x,t_floatarg f)
 
     if(f < 0)
     {
-        pd_error(x,"[comport] ** ERROR ** could not set break of device %s\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4]);
-#else
-            x->serial_device->s_name);
-#endif
+        pd_error(x,"[comport] ** ERROR ** could not set break of device %s\n", x->pretty_name);
     }
-    else comport_verbose("[comport] set break of %s to %g\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4], f);
-#else
-            x->serial_device->s_name, f);
-#endif
+    else comport_verbose("[comport] set break of %s to %g\n", x->pretty_name, f);
 }
 
 static void comport_xonxoff(t_comport *x,t_floatarg f)
@@ -1681,20 +1674,10 @@ static void comport_xonxoff(t_comport *x,t_floatarg f)
 
     if(set_serial(x) == 0)
     {
-        pd_error(x,"[comport] ** ERROR ** could not set xonxoff of device %s\n",
-#ifdef _WIN32
-            &x->serial_device->s_name[4]);
-#else
-            x->serial_device->s_name);
-#endif
+        pd_error(x,"[comport] ** ERROR ** could not set xonxoff of device %s\n", x->pretty_name);
         return;
     }
-    else comport_verbose("[comport] set xonxoff of %s to %g\n",
-#ifdef _WIN32
-        &x->serial_device->s_name[4], f);
-#else
-        x->serial_device->s_name, f);
-#endif
+    else comport_verbose("[comport] set xonxoff of %s to %g\n", x->pretty_name, f);
     x->xonxoff = f;
 }
 
@@ -1712,6 +1695,11 @@ static void comport_close(t_comport *x)
     if (x->x_status_outlet != NULL) outlet_float(x->x_status_outlet, (float)x->comport);
 }
 
+static void comport_lock(t_comport *x, t_floatarg f)
+{
+    x->x_lock = (f != 0);
+}
+
 static void comport_open(t_comport *x, t_floatarg f)
 {
     if(x->comhandle != INVALID_HANDLE_VALUE)
@@ -1720,6 +1708,7 @@ static void comport_open(t_comport *x, t_floatarg f)
     x->comhandle = open_serial(f,x);
 
     clock_delay(x->x_clock, x->x_deltime);
+    check_lock(x);
 }
 
 /*
@@ -1735,6 +1724,7 @@ static void comport_devicename(t_comport *x, t_symbol *s)
 
     x->comhandle = open_serial(USE_DEVICENAME,x);
     clock_delay(x->x_clock, x->x_deltime);
+    check_lock(x);
 }
 
 static void comport_print(t_comport *x, t_symbol *s, int argc, t_atom *argv)
@@ -1757,20 +1747,42 @@ static void comport_print(t_comport *x, t_symbol *s, int argc, t_atom *argv)
     }
 }
 
-static void comport_enum(t_comport *x)
+static void comport_enum(t_comport *x, t_bool to_console, int argc, t_atom *argv)
 {
+    unsigned int    i;
+    t_atom          output_atom[2];
+    t_bool          filter_usb = 0;
+
+    int count;
+    t_symbol *sym_usb = gensym("usb");
+    for(count = 0; count < argc; count++)
+    {
+        if (argv[count].a_type == A_SYMBOL && (atom_getsymbol(&argv[count]) == sym_usb))
+            filter_usb = 1;
+    }
+
 #ifdef _WIN32
     HANDLE          fd;
-    char            device_name[10];
-    unsigned int    i;
+    char            device_name[10];        /* "\\.\COMxx" naming, for CreateFileA() */ 
+    char            simple_device_name[10]; /* "COMxx" naming, for QueryDosDeviceA() */
     DWORD           dw;
     for(i = 1; i < COMPORT_MAX; i++)
     {
 #ifdef _MSC_VER
         sprintf_s(device_name, 10, "%s%d", x->serial_device_prefix, i);
+        sprintf_s(simple_device_name, 10, "COM%d", i);
 #else
         sprintf(device_name, "%s%d", x->serial_device_prefix, i);
+        sprintf(simple_device_name, "COM%d", i);
 #endif
+        char lpTargetPath[5000]; /* buffer to store the path of the COMPORTS */
+        t_bool is_usb = 0;
+        DWORD test = QueryDosDeviceA(simple_device_name, lpTargetPath, 5000);
+        if(test == 0) continue; /* QueryDosDevice returns zero if it didn't find an object */
+        if(!strncmp("\\Device\\USBSER", lpTargetPath, 14))
+            is_usb = 1;
+        if(filter_usb && !is_usb) continue;
+
         fd = CreateFileA( device_name,
                 GENERIC_READ | GENERIC_WRITE,
                 0,
@@ -1783,11 +1795,20 @@ static void comport_enum(t_comport *x)
             dw = GetLastError();
         else
             CloseHandle(fd);
-        if (dw == 0)post("\t%d - COM%d (free)", i, i);
-        else if (dw == ERROR_ACCESS_DENIED)pd_error(x, "\t%d - COM%d (in use)", i, i);
+        if(to_console)
+        {
+            if (dw == 0) post("\t%d - COM%d (free)", i, i);
+            else if (dw == ERROR_ACCESS_DENIED) pd_error(x, "\t%d - COM%d (in use)", i, i);
+        } else {
+            if ((dw == 0)||(dw == ERROR_ACCESS_DENIED))
+            { /* output index and name as a list */
+                SETFLOAT(&output_atom[0], i);
+                SETSYMBOL(&output_atom[1], gensym(&device_name[4]));/* strip the slashes and dot */
+                outlet_anything( x->x_status_outlet, gensym("ports"), 2, output_atom);
+            }
+        }
     }
 #else
-    unsigned int i;
     glob_t         glob_buffer;
     int            fd;
     struct termios test;
@@ -1811,86 +1832,32 @@ static void comport_enum(t_comport *x)
     }
     for(i=0; i<glob_buffer.gl_pathc; i++)
     {
-/* now try to open the device */
-        if((fd = open(glob_buffer.gl_pathv[i], OPENPARAMS)) != INVALID_HANDLE_VALUE)
+        const char *device_name = glob_buffer.gl_pathv[i];
+    /* test the device against the filter */
+        if(filter_usb)
+        {
+#ifdef __APPLE__
+            if(strncmp(device_name, "/dev/tty.usb", 12)) continue;
+#elif defined __linux__
+            if( strncmp(device_name, "/dev/ttyACM", 11)
+                &&
+                strncmp(device_name, "/dev/ttyUSB", 11)
+            ) continue;
+#endif /* TODO: IRIX */
+        }
+    /* now try to open the device */
+        if((fd = open(device_name, OPENPARAMS)) != INVALID_HANDLE_VALUE)
         {
 /* now see if it has attributes */
-            if ((tcgetattr(fd, &test)) != -1)
-                post("\t%d\t%s", i, glob_buffer.gl_pathv[i]);// this one really exists
-            close (fd);
-        }
-    }
-#endif  /* _WIN32 */
-}
-
-static void comport_ports(t_comport *x)
-{ /* the same as comport_enum except outputs list of available ports on status outlet */
-    unsigned int    i;
-    t_atom          output_atom[2];
-#ifdef _WIN32
-    HANDLE          fd;
-    char            device_name[10];
-    DWORD           dw;
-
-    for(i = 1; i < COMPORT_MAX; i++)
-    {
-#ifdef _MSC_VER
-        sprintf_s(device_name, 10, "%s%d", x->serial_device_prefix, i);
-#else
-        sprintf(device_name, "%s%d", x->serial_device_prefix, i);
-#endif
-        fd = CreateFileA( device_name,
-                GENERIC_READ | GENERIC_WRITE,
-                0,
-                0,
-                OPEN_EXISTING,
-                FILE_FLAG_OVERLAPPED,
-                0);
-        dw = 0L;
-        if(fd == INVALID_HANDLE_VALUE)
-            dw = GetLastError();
-        else
-            CloseHandle(fd);
-        if ((dw == 0)||(dw == ERROR_ACCESS_DENIED))
-        { /* output index and name as a list */
-            SETFLOAT(&output_atom[0], i);
-            SETSYMBOL(&output_atom[1], gensym(&device_name[4]));/* strip the slashes and dot */
-            outlet_anything( x->x_status_outlet, gensym("ports"), 2, output_atom);
-        }
-    }
-#else
-    glob_t          glob_buffer;
-    int             fd;
-    struct termios  test;
-
-/* first look for registered devices in the filesystem */
-    switch( glob( x->serial_device_prefix, 0, NULL, &glob_buffer ) )
-    {
-        case GLOB_NOSPACE:
-            pd_error(x,"[comport] out of memory for \"%s\"",x->serial_device_prefix);
-            break;
-# ifdef GLOB_ABORTED
-        case GLOB_ABORTED:
-            pd_error(x,"[comport] aborted \"%s\"",x->serial_device_prefix);
-            break;
-# endif /* GLOB_ABORTED */
-# ifdef GLOB_NOMATCH
-        case GLOB_NOMATCH:
-            pd_error(x,"[comport] no serial devices found for \"%s\"",x->serial_device_prefix);
-            break;
-# endif /* GLOB_NOMATCH */
-    }
-    for(i = 0; (i < glob_buffer.gl_pathc) && (i < COMPORT_MAX); i++)
-    {
-/* now try to open the device */
-        if((fd = open(glob_buffer.gl_pathv[i], OPENPARAMS)) != INVALID_HANDLE_VALUE)
-        {
-/* now see if it has attributes */
-            if ((tcgetattr(fd, &test)) != -1)
-            { /* output index and name as a list */
-                SETFLOAT(&output_atom[0], i);
-                SETSYMBOL(&output_atom[1], gensym(glob_buffer.gl_pathv[i]));
-                outlet_anything( x->x_status_outlet, gensym("ports"), 2, output_atom);
+            if ((tcgetattr(fd, &test)) != -1) {
+                if(to_console)
+                {
+                    post("\t%d\t%s", i, device_name);// this one really exists
+                } else {
+                    SETFLOAT(&output_atom[0], i);
+                    SETSYMBOL(&output_atom[1], gensym(device_name));
+                    outlet_anything( x->x_status_outlet, gensym("ports"), 2, output_atom);
+                }
             }
             close (fd);
         }
@@ -1898,13 +1865,11 @@ static void comport_ports(t_comport *x)
 #endif  /* _WIN32 */
 }
 
-static void comport_output_print(t_comport *x)
+static void comport_ports(t_comport *x, t_symbol *s, int argc, t_atom *argv)
 {
-    post("[comport]: available serial ports:");
-    comport_enum(x);
+    comport_enum(x, 0, argc, argv);
 }
 
-
 static void comport_output_status(t_comport *x, t_symbol *selector, t_float output_value)
 {
     t_atom *output_atom = getbytes(sizeof(t_atom));
@@ -1980,9 +1945,10 @@ static void comport_output_open_status(t_comport *x)
         comport_output_status(x, gensym("open"), 1);
 }
 
-static void comport_devices(t_comport *x)
+static void comport_devices(t_comport *x, t_symbol *s, int argc, t_atom *argv)
 {
-    comport_output_print(x);
+    post("[comport]: available serial ports:");
+    comport_enum(x, 1, argc, argv);
 }
 
 static void comport_info(t_comport *x)
@@ -2019,11 +1985,7 @@ static void comport_help(t_comport *x)
     post("[comport] serial port %d (baud %d):", x->comport, x->baud);
     if(x->comport >= 0 && x->comport < COMPORT_MAX)
     {
-#ifdef WIN32
-        post("\tdevicename: %s", &x->serial_device->s_name[4]);
-#else
-        post("\tdevicename: %s", x->serial_device->s_name);
-#endif
+        post("\tdevicename: %s", x->pretty_name);
     }
 
     post("  Methods:");
@@ -2072,6 +2034,7 @@ void comport_setup(void)
     class_addmethod(comport_class, (t_method)comport_parity, gensym("parity"), A_FLOAT, 0);
     class_addmethod(comport_class, (t_method)comport_xonxoff, gensym("xonxoff"), A_FLOAT, 0);
     class_addmethod(comport_class, (t_method)comport_hupcl, gensym("hupcl"), A_FLOAT, 0);
+    class_addmethod(comport_class, (t_method)comport_lock, gensym("lock"), A_FLOAT, 0);
     class_addmethod(comport_class, (t_method)comport_close, gensym("close"), 0);
     class_addmethod(comport_class, (t_method)comport_open, gensym("open"), A_FLOAT, 0);
     class_addmethod(comport_class, (t_method)comport_devicename, gensym("devicename"), A_SYMBOL, 0);
@@ -2082,14 +2045,14 @@ void comport_setup(void)
     class_addmethod(comport_class, (t_method)comport_set_inprocess, gensym("inputprocess"), A_FLOAT, 0);
     class_addmethod(comport_class, (t_method)comport_help, gensym("help"), 0);
     class_addmethod(comport_class, (t_method)comport_info, gensym("info"), 0);
-    class_addmethod(comport_class, (t_method)comport_devices, gensym("devices"), 0);
-    class_addmethod(comport_class, (t_method)comport_ports, gensym("ports"), 0);
+    class_addmethod(comport_class, (t_method)comport_devices, gensym("devices"), A_GIMME, 0);
+    class_addmethod(comport_class, (t_method)comport_ports, gensym("ports"), A_GIMME, 0);
 
 #ifndef _WIN32
     null_tv.tv_sec = 0; /* no wait */
     null_tv.tv_usec = 0;
 #endif /* NOT _WIN32 */
-    verbose(-1, "comport - PD external for unix/windows\n"
+    logpost(0, PD_DEBUG, "comport - Pd external for unix/windows\n"
         "LGPL-2.1+ 1998-2022,  Winfried Ritsch and others (see LICENSE.txt)\n"
         "Institute for Electronic Music - Graz");
 }
